==> kite-1.0b3.tar.gz: kite-1.0b3/modules/__internal/_regex_match.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "regex/microregex.h"
    
#define TYPECHECK_INT_OPERAND \
    kite_object_t *rhs; \
    if (args->type == OBJ_LIST) { \
        rhs = args->builtin_data.listvalue.car; \
    } else { \
        rhs = args; \
    } \
    kite_object_t *exc; \
    if (rhs->type != OBJ_INTEGER) { \
        exc = kite_new_exception(thd, "System.exceptions.TypeMismatch", \
                            "Second operand must be an integer."); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
        return; \
    }

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(RegexMatch_destruct) {
    microregex_state_destroy((microregex_state_t)this->builtin_data.filevalue);
}

KITE_CLASS_METHOD(RegexMatch_getMatchBegin) {
    kite_object_t *ret;

    ret = kite_new_integer(
        thd,
        microregex_state_get_match_begin(this->builtin_data.filevalue)
    );
    
    kite_vm_return(thd, ret);
}

KITE_CLASS_METHOD(RegexMatch_getGroup) {
    kite_object_t *ret;
    TYPECHECK_INT_OPERAND;

    ret = kite_new_string(
        thd,
        microregex_state_get_subexpression(
            this->builtin_data.filevalue,
            KITE_GET_INTEGER(rhs)
        )
    );
    
    kite_vm_return(thd, ret);
}

KITE_CLASS_METHOD(RegexMatch_getNumGroups) {
    kite_object_t *ret;

    ret = kite_new_integer(
        thd,
        microregex_state_get_num_captured(this->builtin_data.filevalue)
    );

    kite_vm_return(thd, ret);
}
    
KITE_MODULE_INITIALIZER(_regex_match) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_regex_match");
    
    /* Add methods. */
    kite_add_method(thread, newclass, "get_match_begin", 
        kite_new_method(thread, FUNC_COMPILED, RegexMatch_getMatchBegin, 0));
    kite_add_method(thread, newclass, "get_num_groups", 
        kite_new_method(thread, FUNC_COMPILED, RegexMatch_getNumGroups, 0));
    kite_add_method(thread, newclass, "get_group", 
        kite_new_method(thread, FUNC_COMPILED, RegexMatch_getGroup, 1));
    newclass->object_data.destructor =
        kite_new_method(thread, FUNC_COMPILED, RegexMatch_destruct, 0);
            
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_regex_match", 1);
    kite_set_property(thread, parent, "_regex_match", newclass);
}

==> kite-1.0.0.tar.gz: kite-1.0.0/modules/__internal/_regex_match.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "regex/microregex.h"
    
#define TYPECHECK_INT_OPERAND \
    kite_object_t *rhs; \
    kite_object_t *exc; \
    if (args->type == OBJ_LIST) { \
        rhs = args->builtin_data.listvalue.car; \
    } else { \
        rhs = args; \
    } \
    if (rhs->type != OBJ_INTEGER) { \
        exc = kite_new_exception(thd, "System.exceptions.TypeMismatch", \
                            "Second operand must be an integer."); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
        return; \
    }

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(RegexMatch_destruct) {
    (void)thd;
    KITE_NO_ARGS;
    microregex_state_destroy((microregex_state_t*)&this->builtin_data.filevalue);
}

KITE_CLASS_METHOD(RegexMatch_getMatchBegin) {
    kite_object_t *ret;

    KITE_NO_ARGS;
    
    ret = kite_new_integer(
        thd,
        microregex_state_get_match_begin(this->builtin_data.filevalue)
    );
    
    kite_vm_return(thd, ret);
}

KITE_CLASS_METHOD(RegexMatch_getGroup) {
    kite_object_t *ret;
    TYPECHECK_INT_OPERAND;

    ret = kite_new_string(
        thd,
        microregex_state_get_subexpression(
            this->builtin_data.filevalue,
            KITE_GET_INTEGER(rhs)
        )
    );
    
    kite_vm_return(thd, ret);
}

KITE_CLASS_METHOD(RegexMatch_getNumGroups) {
    kite_object_t *ret;
    KITE_NO_ARGS;
    
    ret = kite_new_integer(
        thd,
        microregex_state_get_num_captured(this->builtin_data.filevalue)
    );

    kite_vm_return(thd, ret);
}
    
KITE_MODULE_INITIALIZER(_regex_match) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_regex_match");
    kite_set_docstring(newclass, "Regex matching code internal code.");
    
    /* Add methods. */
    kite_add_method(thread, newclass, "get_match_begin", 
        kite_new_method_compiled_with_docs(thread, RegexMatch_getMatchBegin, 
                                  "Get 0-based position in string where match occurred.", 0));
    kite_add_method(thread, newclass, "get_num_groups", 
        kite_new_method_compiled_with_docs(thread, RegexMatch_getNumGroups, 
                                  "Get the number of subgroups collected.", 0));
    kite_add_method(thread, newclass, "get_group", 
        kite_new_method_compiled_with_docs(thread, RegexMatch_getGroup, 
                                  "Get the subgroup's matched text.", 1,
                                  "group", "Group number to retrieve (0 is the entire match.)"));
    kite_add_method(thread, newclass, "__destruct__",
        kite_new_method_compiled_with_docs(thread, RegexMatch_destruct, 
                                  "Class destructor.", 0));
            
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_regex_match", 1, NULL);
    kite_set_property(thread, parent, "_regex_match", newclass);
}
